<?php
/**
* Plugin Name: miniOrange OAuth 2.0 Server/Provider
* Plugin URI: http://miniorange.com
* Description: Setup your site as Identity Server to allow Login with WordPress or WordPress Login to other client application /site using OAuth / OpenID Connect protocols.
* Version: 3.0.4
* Author: miniOrange
* Author URI: https://www.miniorange.com
* License: MIT/Expat
* License URI: https://docs.miniorange.com/mit-license
*/
define( 'MOC_DIR', plugin_dir_path( __FILE__ ) );
define( 'MOC_URL', plugin_dir_url( __FILE__ ) );
include_once dirname( __FILE__ ) . '/class-mo-oauth-widget.php';
require('feedback_form.php');
require('class-customer.php');
require('mo_oauth_settings_page.php');
require('mo_oauth_db_handler.php');
// require( 'endpoints/registry.php' );

class mo_oauth_server {

	function __construct() {
		add_action( 'admin_menu', array( $this, 'miniorange_menu' ) );
		add_action( 'admin_init',  array( $this, 'miniorange_oauth_save_settings' ) );
		add_action( 'plugins_loaded',  array( $this, 'mo_login_widget_text_domain' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'plugin_settings_style' ) );
		register_deactivation_hook(__FILE__, array( $this, 'mo_oauth_deactivate'));
		register_activation_hook(__FILE__, array( $this, 'mo_oauth_activate'));
		add_action( 'admin_enqueue_scripts', array( $this, 'plugin_settings_script' ) );
		//add_action( 'admin_enqueue_scripts', array( $this, 'tutorial' ) );
		remove_action( 'admin_notices', array( $this, 'mo_oauth_success_message') );
		remove_action( 'admin_notices', array( $this, 'mo_oauth_error_message') );
		add_shortcode('mo_oauth_login', array( $this,'mo_oauth_shortcode_login'));
		add_action( 'admin_footer', array( $this, 'mo_oauth_server_feedback_request' ) );	
	}
	function tutorial($page) {
		$file = plugin_dir_path( __FILE__ ) . 'pointers.php';
		// Arguments: pointers php file, version (dots will be replaced), prefix
		$manager = new MOOAuthServerPointersManager( $file, '2.5.1', 'custom_admin_pointers' );
		// print_r($manager);
		$manager->parse();
		$pointers = $manager->filter( $page );
		if ( empty( $pointers ) ) { // nothing to do if no pointers pass the filter
			return;
		}
		wp_enqueue_style( 'wp-pointer' );
		$js_url = plugins_url( 'cards.js', __FILE__ );
		wp_enqueue_script( 'custom_admin_pointers', $js_url, array('wp-pointer'), NULL, TRUE );
		// data to pass to javascript
		$data = array(
			'next_label' => __( 'Next' ),
			'close_label' => __('Close'),
			'pointers' => $pointers
		);

		wp_localize_script( 'custom_admin_pointers', 'MyAdminPointers', $data );

	}

	function mo_oauth_success_message() {
		$class = "error";
		$message = get_option('message');
		echo "<div class='" . $class . "'> <p>" . $message . "</p></div>";
	}

	function mo_oauth_server_feedback_request() {
		mo_oauth_server_display_feedback_form();
	}

	function mo_oauth_error_message() {
		$class = "updated";
		$message = get_option('message');
		echo "<div class='" . $class . "'><p>" . $message . "</p></div>";
	}

	public function mo_oauth_activate() {
		$moOauthServerDb = new MoOauthServerDb();
		$moOauthServerDb->mo_plugin_activate();
	}

	public function mo_oauth_deactivate() {
		//delete all stored key-value pairs
		delete_option('host_name');
		delete_option('mo_oauth_server_new_registration');
		delete_option('mo_oauth_server_admin_phone');
		delete_option('mo_oauth_server_verify_customer');
		delete_option('mo_oauth_server_admin_customer_key');
		delete_option('mo_oauth_server_admin_api_key');
		delete_option('mo_oauth_server_new_customer');
		delete_option('mo_oauth_server_customer_token');
		delete_option('message');
		delete_option('mo_oauth_server_registration_status');
		delete_option('mo_oauth_show_mo_server_message');
	}

	private $settings = array(
		'mo_oauth_facebook_client_secret'	=> '',
		'mo_oauth_facebook_client_id' 		=> '',
		'mo_oauth_facebook_enabled' 		=> 0
	);

	function miniorange_menu() {

		//Add miniOrange plugin to the menu
		$page = add_menu_page( 'MO OAuth Settings ' . __( 'Configure OAuth', 'mo_oauth_server_settings' ), 'miniOrange OAuth Server', 'administrator', 'mo_oauth_server_settings', array( $this, 'mo_oauth_login_options' ) ,plugin_dir_url(__FILE__) . 'images/miniorange.png');


	}

	function  mo_oauth_login_options () {
		global $wpdb;
		update_option( 'host_name', 'https://login.xecurify.com', false );
		$customerRegistered = mo_oauth_server_is_customer_registered();
		if( $customerRegistered ) {
			mo_oauth_server_page();
		} else {
			mo_oauth_server_page();
		}
	}

	function plugin_settings_style($hook) {
		wp_enqueue_style( 'mo_oauth_admin_feedback_form_settings_style', plugins_url( 'css/feedback_form_style_settings.css', __FILE__ ) );
		if($hook != 'toplevel_page_mo_oauth_server_settings') {
            return;
        }
		wp_enqueue_style( 'mo_oauth_admin_settings_style', plugins_url( 'style_settings.css', __FILE__ ), array(), '3.0.0');
		wp_enqueue_style( 'mo_oauth_admin_settings_phone_style', plugins_url( 'phone.css', __FILE__ ) );
		wp_enqueue_style( 'mo_oauth_admin_font_awesome', plugins_url( 'css/font-awesome.css', __FILE__ ) );
		// if(isset($_REQUEST['tab']) && $_REQUEST['tab'] == 'licensing'){
  //           wp_enqueue_style( 'mo_oauth_bootstrap_css', plugins_url( 'css/bootstrap/bootstrap.min.css', __FILE__ ) );
  //       }
	}

	function plugin_settings_script($hook) {
		if($hook != 'toplevel_page_mo_oauth_server_settings') {
            return;
        }
		wp_enqueue_script( 'mo_oauth_admin_settings_script', plugins_url( 'settings.js', __FILE__ ) );
		wp_enqueue_script( 'mo_oauth_admin_settings_phone_script', plugins_url('phone.js', __FILE__ ) );
		if(isset($_REQUEST['tab']) && $_REQUEST['tab'] == 'licensing'){
            wp_enqueue_script( 'mo_oauth_modernizr_script', plugins_url( 'js/modernizr.js', __FILE__ ) );
            wp_enqueue_script( 'mo_oauth_popover_script', plugins_url( 'js/bootstrap/popper.min.js', __FILE__ ) );
            wp_enqueue_script( 'mo_oauth_bootstrap_script', plugins_url( 'js/bootstrap/bootstrap.min.js', __FILE__ ) );
        }
	}

	function mo_login_widget_text_domain() {
		load_plugin_textdomain( 'flw', FALSE, basename( dirname( __FILE__ ) ) . '/languages' );
		// to update the database on plugin update
		$this->plugin_update();
	}

	function plugin_update(){
		global $wpdb;
		// update the table moos_oauth_clients
		$table_name = $wpdb->base_prefix."moos_oauth_clients";
        $row = $wpdb->get_results( "SELECT COLUMN_NAME FROM INFORMATION_SCHEMA.COLUMNS WHERE table_schema = '" . DB_NAME ."' AND table_name = '". $table_name . "' AND column_name ='active_oauth_server_id'", ARRAY_A);
		if ( empty($row) ) {
			$sql = "ALTER TABLE ".$wpdb->base_prefix."moos_oauth_clients ADD active_oauth_server_id INT DEFAULT ".get_current_blog_id();
			$wpdb->query($sql);
		}
	}

	private function mo_oauth_show_success_message() {
		remove_action( 'admin_notices', array( $this, 'mo_oauth_success_message') );
		add_action( 'admin_notices', array( $this, 'mo_oauth_error_message') );
	}

	private function mo_oauth_show_error_message() {
		remove_action( 'admin_notices', array( $this, 'mo_oauth_error_message') );
		add_action( 'admin_notices', array( $this, 'mo_oauth_success_message') );
	}

	public function mo_oauth_check_empty_or_null( $value ) {
		if( ! isset( $value ) || empty( $value ) ) {
			return true;
		}
		return false;
	}

	function miniorange_oauth_save_settings(){

		if( ! session_id() ) {
			session_start(['read_and_close' => true,]);
		}

		if ( isset( $_POST['option'] ) and $_POST['option'] == "mo_oauth_mo_server_message" ) {
			update_option( 'mo_oauth_show_mo_server_message', 1 , false);

			return;
		}

		if ( isset( $_POST['option'] ) and $_POST['option'] == "clear_pointers" ) {
			update_user_meta(get_current_user_id(),'dismissed_wp_pointers','');
			return;
		}

		if(isset($_POST['option']) and $_POST['option'] == "change_miniorange") {
			delete_option('host_name');
			delete_option('mo_oauth_admin_email');
			delete_option('mo_oauth_server_admin_phone');
			delete_option('mo_oauth_server_verify_customer');
			delete_option('mo_oauth_server_admin_customer_key');
			delete_option('mo_oauth_server_admin_api_key');
			delete_option('mo_oauth_server_customer_token');
			delete_option('mo_oauth_server_new_customer');
			delete_option('message');
			delete_option('mo_eve_api_key');
			delete_option('mo_oauth_server_new_registration');
			delete_option('mo_oauth_server_registration_status');
			delete_option('mo_oauth_show_mo_server_message');
			return;
		}

		if( isset( $_POST['option'] ) and $_POST['option'] == "mo_oauth_register_customer" ) {	//register the admin to miniOrange
			//validation and sanitization
			$email = '';
			$phone = '';
			$password = '';
			$confirmPassword = '';
			$fname = '';
			$lname = '';
			$company = '';
			if( $this->mo_oauth_check_empty_or_null( $_POST['email'] ) || $this->mo_oauth_check_empty_or_null( $_POST['password'] ) || $this->mo_oauth_check_empty_or_null( $_POST['confirmPassword'] ) ) {
				update_option( 'message', 'All the fields are required. Please enter valid entries.', false);
				$this->mo_oauth_show_error_message();
				return;
			} else if( strlen( $_POST['password'] ) < 8 || strlen( $_POST['confirmPassword'] ) < 8){
				update_option( 'message', 'Choose a password with minimum length 8.', false);
				$this->mo_oauth_show_error_message();
				return;
			} else{

				$email = sanitize_email( $_POST['email'] );
				$password = stripslashes( $_POST['password'] );
				$confirmPassword = stripslashes( $_POST['confirmPassword'] );
				$fname = stripslashes( $_POST['fname'] );
				$lname = stripslashes( $_POST['lname'] );
				$company = stripslashes( $_POST['company'] );
			}

			update_option( 'mo_oauth_admin_email', $email , false);
			update_option( 'mo_oauth_admin_fname', $fname , false);
			update_option( 'mo_oauth_admin_lname', $lname, false );
			update_option( 'mo_oauth_admin_company', $company , false);

			if( mo_oauth_server_is_curl_installed() == 0 ) {
				return $this->mo_oauth_show_curl_error();
			}

			if( strcmp( $password, $confirmPassword) == 0 ) {
				$email = get_option('mo_oauth_admin_email');
				update_option( 'password', $password, false );
				$customer = new Mo_Auth_Server_Customer();
				$content = json_decode($customer->check_customer(), true);

				if( strcasecmp( $content['status'], 'CUSTOMER_NOT_FOUND') == 0 ) {
					$response = json_decode($customer->create_customer(), true);
					if(strcasecmp($response['status'], 'SUCCESS') == 0) {
						$content = $customer->get_customer_key();
						$customerKey = json_decode( $content, true );
							if( json_last_error() == JSON_ERROR_NONE ) {
								update_option( 'mo_oauth_server_admin_customer_key', $customerKey['id'], false );
								update_option( 'mo_oauth_server_admin_api_key', $customerKey['apiKey'] , false);
								update_option( 'mo_oauth_server_customer_token', $customerKey['token'] , false);
								update_option( 'mo_oauth_server_admin_phone', $customerKey['phone'] , false);
								delete_option( 'password' );
								update_option( 'message', 'Customer created & retrieved successfully', false);
								delete_option( 'mo_oauth_server_verify_customer' );
								$this->mo_oauth_show_success_message();
							}
						wp_redirect( admin_url( '/admin.php?page=mo_oauth_server_settings&tab=login' ), 301 );
						exit;
					} else {
						update_option( 'message', 'Failed to create customer. Try again.', false);
					}
					$this->mo_oauth_show_success_message();
				} elseif(strcasecmp( $content['status'], 'SUCCESS') == 0 ) {
					update_option( 'message', 'Account already exist. Please Login.', false);
				} else {
					update_option( 'message', $content['status'] , false);
				}
				$this->mo_oauth_show_success_message();

			} else {
				update_option( 'message', 'Passwords do not match.', false);
				delete_option('mo_oauth_server_verify_customer');
				$this->mo_oauth_show_error_message();
			}
		}

		if(isset($_POST['option']) and $_POST['option'] == "mo_oauth_register"){
			update_option('goto_registration', true, false);
			return;
		}
		else if(isset($_POST['option']) and $_POST['option'] == "mo_oauth_validate_otp"){
			if( mo_oauth_server_is_curl_installed() == 0 ) {
				return $this->mo_oauth_show_curl_error();
			}
			//validation and sanitization
			$otp_token = '';
			if( $this->mo_oauth_check_empty_or_null( $_POST['mo_oauth_otp_token'] ) ) {
				update_option( 'message', 'Please enter a value in OTP field.', false);
				update_option('mo_oauth_server_registration_status','MO_OTP_VALIDATION_FAILURE', false);
				$this->mo_oauth_show_error_message();
				return;
			} else{
				$otp_token = stripslashes( $_POST['mo_oauth_otp_token'] );
			}

			$customer = new Mo_Auth_Server_Customer();
			$content = json_decode($customer->validate_otp_token($_SESSION['mo_oauth_transactionId'], $otp_token ),true);
			if(strcasecmp($content['status'], 'SUCCESS') == 0) {
				$this->create_customer();
			}else{
				update_option( 'message','Invalid one time passcode. Please enter a valid OTP.', false);
				update_option('mo_oauth_server_registration_status','MO_OTP_VALIDATION_FAILURE', false);
				$this->mo_oauth_show_error_message();
			}
		}

		if( isset( $_POST['option'] ) and $_POST['option'] == "mo_oauth_goto_login" ) {
			delete_option( 'mo_oauth_server_new_registration');
			update_option( 'mo_oauth_server_verify_customer', 'true' , false);
		}

		if( isset( $_POST['option'] ) and $_POST['option'] == "mo_oauth_verify_customer" ) {	//register the admin to miniOrange

			if( mo_oauth_server_is_curl_installed() == 0 ) {
				return $this->mo_oauth_show_curl_error();
			}
			//validation and sanitization
			$email = '';
			$password = '';
			if( $this->mo_oauth_check_empty_or_null( $_POST['email'] ) || $this->mo_oauth_check_empty_or_null( $_POST['password'] ) ) {
				update_option( 'message', 'All the fields are required. Please enter valid entries.', false);
				$this->mo_oauth_show_error_message();
				return;
			} else {
				$email = sanitize_email( $_POST['email'] );
				$password = stripslashes( $_POST['password'] );
			}

			update_option( 'mo_oauth_admin_email', $email , false);
			update_option( 'password', $password , false);
			$customer = new Mo_Auth_Server_Customer();
			$content = $customer->get_customer_key();
			$customerKey = json_decode( $content, true );
			if( json_last_error() == JSON_ERROR_NONE ) {
				update_option( 'mo_oauth_server_admin_customer_key', $customerKey['id'] , false);
				update_option( 'mo_oauth_server_admin_api_key', $customerKey['apiKey'], false );
				update_option( 'mo_oauth_server_customer_token', $customerKey['token'] , false);
				update_option( 'mo_oauth_server_admin_phone', $customerKey['phone'] , false);
				delete_option( 'password' );
				update_option( 'message', 'Customer retrieved successfully', false);
				delete_option( 'mo_oauth_server_verify_customer' );
				$this->mo_oauth_show_success_message();
			} else {
				update_option( 'message', 'Invalid username or password. Please try again.', false);
				$this->mo_oauth_show_error_message();
			}
		} else if( isset( $_POST['option'] ) and $_POST['option'] == "mo_oauth_add_client" ) {
			$clientid = '';
			$clientsecret = '';
			if($this->mo_oauth_check_empty_or_null($_POST['mo_oauth_custom_client_name']) || $this->mo_oauth_check_empty_or_null($_POST['mo_oauth_client_redirect_url'])) {
				update_option( 'message', 'Please enter valid Client Name and Redirect URI.', false);
				$this->mo_oauth_show_error_message();
				return;
			} else{
				$client_name = sanitize_text_field( $_POST['mo_oauth_custom_client_name'] );
				$redirect_url = sanitize_text_field( $_POST['mo_oauth_client_redirect_url'] );
				if ( empty( $client_name ) || empty( $redirect_url ) ) {
					update_option( 'message', 'Please enter valid Client Name and Redirect URI.', false);
					$this->mo_oauth_show_error_message();
					return;
				}
				$active_oauth_server_id = get_current_blog_id();
				$moOauthServerDb = new MoOauthServerDb();
				$clientlist = $moOauthServerDb->get_clients();
				if(sizeof($clientlist) < 1) {
					$moOauthServerDb->add_client($client_name, moosGenerateRandomString(32), $redirect_url, $active_oauth_server_id);
					update_option( 'message', 'Your settings are saved successfully.' , false);
					update_option('mo_oauth_server_client',sanitize_text_field($_POST['client']), false);
					$this->mo_oauth_show_success_message();
				}
				else {
					update_option( 'message', 'You can add only 1 client with free version.' , false);
					$this->mo_oauth_show_error_message();
				}
			}
		}
		else if( isset( $_POST['option'] ) and $_POST['option'] == "mo_oauth_update_client" ) {
			$clientid = '';
			$clientsecret = '';
			if($this->mo_oauth_check_empty_or_null($_POST['mo_oauth_custom_client_name']) || $this->mo_oauth_check_empty_or_null($_POST['mo_oauth_client_redirect_url'])) {
				update_option( 'message', 'Please enter valid Client Name and Redirect URI.', false);
				$this->mo_oauth_show_error_message();
				return;
			} else{
				$client_name = stripslashes( $_POST['mo_oauth_custom_client_name'] );
				$redirect_url = stripslashes( $_POST['mo_oauth_client_redirect_url'] );
				$moOauthServerDb = new MoOauthServerDb();
				$moOauthServerDb->update_client($client_name, $redirect_url);
				update_option( 'message', 'Your settings are saved successfully.', false );
				$this->mo_oauth_show_success_message();
			}
		}
		else if( isset( $_POST['option'] ) and $_POST['option'] == "mo_oauth_attribute_mapping" ) {
			$appname = stripslashes( $_POST['mo_oauth_app_name'] );
			$email_attr = stripslashes( $_POST['mo_oauth_email_attr'] );
			$name_attr = stripslashes( $_POST['mo_oauth_name_attr'] );

			$appslist = get_option('mo_oauth_apps_list');
			foreach($appslist as $key => $currentapp){
				if($appname == $key){
					$currentapp['email_attr'] = $email_attr;
					$currentapp['name_attr'] = $name_attr;
					$appslist[$key] = $currentapp;
					break;
				}
			}
			update_option('mo_oauth_apps_list', $appslist, false);
			update_option( 'message', 'Your settings are saved successfully.' , false);
			$this->mo_oauth_show_success_message();
			wp_redirect('admin.php?page=mo_oauth_server_settings&tab=config&action=update&app='.urlencode($appname));
		} else if(isset($_POST['option']) and $_POST['option'] == "mo_oauth_server_master_switch") {
			if(!isset($_POST['mo_server_master_switch'])) {
				update_option('mo_oauth_server_master_switch', 'off', false);
			} else {
				update_option('mo_oauth_server_master_switch', 'on', false);
			}
			update_option( 'message', 'Your settings are saved successfully.', false );
			$this->mo_oauth_show_success_message();
			wp_redirect('admin.php?page=mo_oauth_server_settings&tab=general_settings');
		} else if(isset($_POST['option']) and $_POST['option'] == "mo_oauth_general_settings") {
			if(isset($_POST['mo_server_token_length'])) {
				update_option('mo_oauth_server_token_length', (int) stripslashes($_POST["mo_server_token_length"]), false);
			}

			if(isset($_POST['expiry_time'])) {
				update_option( 'mo_oauth_expiry_time', intval(sanitize_text_field($_POST['expiry_time'])), false);
			}

			if(isset($_POST['refresh_expiry_time'])) {
				update_option( 'mo_oauth_refresh_expiry_time', intval(sanitize_text_field($_POST['refresh_expiry_time'])), false);
			}

			update_option( 'message', 'Your settings are saved successfully.', false );
			$this->mo_oauth_show_success_message();
			wp_redirect('admin.php?page=mo_oauth_server_settings&tab=general_settings');
		} else if(isset($_POST['option']) and $_POST['option'] == "mo_oauth_server_enforce_state") {
			if(!isset($_POST['mo_server_enforce_state'])) {
				update_option('mo_oauth_server_enforce_state', 'off', false);
			} else {
				update_option('mo_oauth_server_enforce_state', 'on', false);
			}

			update_option( 'message', 'Your settings are saved successfully.', false );
			$this->mo_oauth_show_success_message();
			wp_redirect('admin.php?page=mo_oauth_server_settings&tab=general_settings');
		} else if(isset($_POST['option']) and $_POST['option'] == "mo_oauth_server_enable_oidc") {
			if(!isset($_POST['mo_server_enable_oidc'])) {
				update_option('mo_oauth_server_enable_oidc', 'off', false);
			} else {
				update_option('mo_oauth_server_enable_oidc', 'on', false);
			}

			update_option( 'message', 'Your settings are saved successfully.', false );
			$this->mo_oauth_show_success_message();
			wp_redirect('admin.php?page=mo_oauth_server_settings&tab=advanced_settings');
		}
		//submit google form
		else if( isset( $_POST['option'] ) and $_POST['option'] == "mo_oauth_google" ) {
			if( mo_oauth_server_is_curl_installed() == 0 ) {
				return $this->mo_oauth_show_curl_error();
			}
			//validation and sanitization
			$scope = '';
			$clientid = '';
			$clientsecret = '';
			if($this->mo_oauth_check_empty_or_null($_POST['mo_oauth_google_scope']) || $this->mo_oauth_check_empty_or_null($_POST['mo_oauth_google_client_id']) || $this->mo_oauth_check_empty_or_null($_POST['mo_oauth_google_client_secret'])) {
				update_option( 'message', 'Please enter Client ID and Client Secret to save settings.', false);
				$this->mo_oauth_show_error_message();
				return;
			} else{
				$scope = stripslashes( $_POST['mo_oauth_google_scope'] );
				$clientid = stripslashes( $_POST['mo_oauth_google_client_id'] );
				$clientsecret = stripslashes( $_POST['mo_oauth_google_client_secret'] );
			}

			if(mo_oauth_server_is_customer_registered()) {
				update_option( 'mo_oauth_google_enable', isset( $_POST['mo_oauth_google_enable']) ? $_POST['mo_oauth_google_enable'] : 0, false);
				update_option( 'mo_oauth_google_scope', $scope, false);
				update_option( 'mo_oauth_google_client_id', $clientid, false);
				update_option( 'mo_oauth_google_client_secret', $clientsecret, false);
				if(get_option('mo_oauth_google_client_id') && get_option('mo_oauth_google_client_secret')) {
					$customer = new Mo_Auth_Server_Customer();
					$message = $customer->add_oauth_application( 'google', 'Google OAuth' );
					if($message == 'Application Created') {
						update_option( 'message', 'Your settings were saved' , false);
						$this->mo_oauth_show_success_message();
					} else {
						update_option( 'message', $message , false);
						$this->mo_oauth_show_error_message();
					}
				} else {
					update_option( 'message', 'Please enter Client ID and Client Secret to save settings', false);
					update_option( 'mo_oauth_google_enable', false, false);
					$this->mo_oauth_show_error_message();
				}
			} else {
				update_option('message', 'Please register customer before trying to save other configurations', false);
				$this->mo_oauth_show_error_message();
			}
		}
		//submit eveonline form
		else if(isset($_POST['option']) and $_POST['option'] == "mo_oauth_eveonline"){
			if( mo_oauth_server_is_curl_installed() == 0 ) {
				return $this->mo_oauth_show_curl_error();
			}
			//validation and sanitization
			$clientid = '';
			$clientsecret = '';
			if($this->mo_oauth_check_empty_or_null($_POST['mo_oauth_eveonline_client_secret']) || $this->mo_oauth_check_empty_or_null($_POST['mo_oauth_eveonline_client_secret'])) {
				update_option( 'message', 'Please enter Client ID and Client Secret to save settings.', false);
				$this->mo_oauth_show_error_message();
				return;
			} else{
				$clientid = stripslashes($_POST['mo_oauth_eveonline_client_id']);
				$clientsecret = stripslashes($_POST['mo_oauth_eveonline_client_secret']);
			}

			if(mo_oauth_server_is_customer_registered()) {
				update_option( 'mo_oauth_eveonline_enable', isset($_POST['mo_oauth_eveonline_enable']) ? $_POST['mo_oauth_eveonline_enable'] : 0, false);
				update_option( 'mo_oauth_eveonline_client_id', $clientid, false);
				update_option( 'mo_oauth_eveonline_client_secret', $clientsecret, false);
				if(get_option('mo_oauth_eveonline_client_id') && get_option('mo_oauth_eveonline_client_secret')) {
					$customer = new Mo_Auth_Server_Customer();
					$message = $customer->add_oauth_application('eveonline', 'EVE Online OAuth');
					if($message == 'Application Created') {
						update_option('message', 'Your settings were saved. Go to Advanced EVE Online Settings for configuring restrictions on user sign in.', false);
						$this->mo_oauth_show_success_message();
					} else {
						update_option('message', $message, false);
						$this->mo_oauth_show_error_message();
					}
				} else {
					update_option( 'message', 'Please enter Client ID and Client Secret to save settings', false);
					update_option( 'mo_oauth_eveonline_enable', false, false);
					$this->mo_oauth_show_error_message();
				}
			} else {
				update_option('message', 'Please register customer before trying to save other configurations', false);
				$this->mo_oauth_show_error_message();
			}
		}
		// submit facebook app
		else if( isset( $_POST['option'] ) and $_POST['option'] == "mo_oauth_facebook" ) {
			if( mo_oauth_server_is_curl_installed() == 0 ) {
				return $this->mo_oauth_show_curl_error();
			}
			//validation and sanitization
			$scope = '';
			$clientid = '';
			$clientsecret = '';
			if($this->mo_oauth_check_empty_or_null($_POST['mo_oauth_facebook_scope']) || $this->mo_oauth_check_empty_or_null($_POST['mo_oauth_facebook_client_id']) || $this->mo_oauth_check_empty_or_null($_POST['mo_oauth_facebook_client_secret'])) {
				update_option( 'message', 'Please enter Client ID and Client Secret to save settings.', false);
				$this->mo_oauth_show_error_message();
				return;
			} else{
				$scope = stripslashes( $_POST['mo_oauth_facebook_scope'] );
				$clientid = stripslashes( $_POST['mo_oauth_facebook_client_id'] );
				$clientsecret = stripslashes( $_POST['mo_oauth_facebook_client_secret'] );
			}

			if(mo_oauth_server_is_customer_registered()) {
				update_option( 'mo_oauth_facebook_enable', isset( $_POST['mo_oauth_facebook_enable']) ? $_POST['mo_oauth_facebook_enable'] : 0, false);
				update_option( 'mo_oauth_facebook_scope', $scope, false);
				update_option( 'mo_oauth_facebook_client_id', $clientid, false);
				update_option( 'mo_oauth_facebook_client_secret', $clientsecret, false);
				if(get_option('mo_oauth_facebook_client_id') && get_option('mo_oauth_facebook_client_secret')) {
					$customer = new Mo_Auth_Server_Customer();
					$message = $customer->add_oauth_application( 'facebook', 'Facebook OAuth' );
					if($message == 'Application Created') {
						update_option( 'message', 'Your settings were saved' , false);
						$this->mo_oauth_show_success_message();
					} else {
						update_option( 'message', $message, false );
						$this->mo_oauth_show_error_message();
					}
				} else {
					update_option( 'message', 'Please enter Client ID and Client Secret to save settings', false);
					update_option( 'mo_oauth_google_enable', false, false);
					$this->mo_oauth_show_error_message();
				}
			} else {
				update_option('message', 'Please register customer before trying to save other configurations', false);
				$this->mo_oauth_show_error_message();
			}
		}
		elseif( isset( $_POST['option'] ) and $_POST['option'] == "mo_oauth_contact_us_query_option" ) {
			if( mo_oauth_server_is_curl_installed() == 0 ) {
				return $this->mo_oauth_show_curl_error();
			}
			// Contact Us query
			$email = stripslashes($_POST['mo_oauth_contact_us_email']);
			$phone = stripslashes($_POST['mo_oauth_contact_us_phone']);
			$query = stripslashes($_POST['mo_oauth_contact_us_query']);

			if(array_key_exists('mo_idp_upgrade_plan_name',$_POST)) {
				$plan_name 	= sanitize_text_field($_POST['mo_idp_upgrade_plan_name']);
				$plan_users = sanitize_text_field($_POST['mo_idp_upgrade_plan_users']);
				$query = "Plan Name : ".$plan_name.", Users : ".$plan_users.", ".$query;
			}

			$customer = new Mo_Auth_Server_Customer();
			if ( $this->mo_oauth_check_empty_or_null( $email ) || $this->mo_oauth_check_empty_or_null( $query ) ) {
				update_option('message', 'Please fill up Email and Query fields to submit your query.', false);
				$this->mo_oauth_show_error_message();
			} else {
				$submited = $customer->submit_contact_us( $email, $phone, $query );
				if ( $submited == false ) {
					update_option('message', 'Your query could not be submitted. Please try again.', false);
					$this->mo_oauth_show_error_message();
				} else {
					update_option('message', 'Thanks for getting in touch! We shall get back to you shortly.', false);
					$this->mo_oauth_show_success_message();
				}
			}
		}
		else if( isset( $_POST['option'] ) and $_POST['option'] == "mo_oauth_resend_otp" ) {
			$email = get_option('mo_oauth_admin_email');
			if( mo_oauth_server_is_curl_installed() == 0 ) {
				return $this->mo_oauth_show_curl_error();
			}
			$customer = new Mo_Auth_Server_Customer();
			$content = json_decode($customer->send_otp_token($email,''), true);
			if(strcasecmp($content['status'], 'SUCCESS') == 0) {
					update_option( 'message', ' A one time passcode is sent to ' . get_option('mo_oauth_admin_email') . ' again. Please check if you got the otp and enter it here.', false);
					$_SESSION['mo_oauth_transactionId'] = $content['txId'];
					update_option('mo_oauth_server_registration_status','MO_OTP_DELIVERED_SUCCESS', false);
					$this->mo_oauth_show_success_message();
			}else{
					update_option('message','There was an error in sending email. Please click on Resend OTP to try again.', false);
					update_option('mo_oauth_server_registration_status','MO_OTP_DELIVERED_FAILURE', false);
					$this->mo_oauth_show_error_message();
			}
		}
		else if( isset( $_POST['option'] ) and $_POST['option'] == "mo_oauth_change_email" ) {
			update_option('mo_oauth_server_verify_customer', '', false);
			update_option('mo_oauth_server_registration_status','', false);
			update_option('mo_oauth_server_new_registration','true', false);
		}
		else if( isset( $_POST['option'] ) and $_POST['option'] == "mo_oauth_register_with_phone_option") {
			$phone = stripslashes($_POST['phone']);
			$phone = str_replace(' ', '', $phone);
			$phone = str_replace('-', '', $phone);
			update_option('mo_oauth_server_admin_phone', $phone, false);
			$customer = new Mo_Auth_Server_Customer();
			$content = json_decode($customer->send_otp_token('', $phone, FALSE, TRUE), true);
			if(strcasecmp($content['status'], 'SUCCESS') == 0) {
				update_option( 'message', ' A one time passcode is sent to ' . get_option('mo_oauth_server_admin_phone') . '. Please enter the otp here to verify your phone.', false);
				$_SESSION['mo_oauth_transactionId'] = $content['txId'];
				// print_r($_SESSION);exit;
				update_option('mo_oauth_server_registration_status','MO_OTP_DELIVERED_SUCCESS_PHONE', false);
				$this->mo_oauth_show_success_message();
			} else {
				update_option('message','There was an error in sending SMS. Please click on Resend OTP to try again.', false);
				update_option('mo_oauth_server_registration_status','MO_OTP_DELIVERED_FAILURE_PHONE', false);
				$this->mo_oauth_show_error_message();
			}
		} else if( isset( $_POST['option'] ) and $_POST['option'] == "mo_oauth_change_phone" ) {
			update_option('mo_oauth_server_registration_status','', false);
		}
		if ( isset( $_POST['option'] ) and $_POST['option'] == 'mo_oauth_server_skip_feedback' ) {
			deactivate_plugins( __FILE__ );
			update_option( 'message', 'Plugin deactivated successfully' , false);
			$this->mo_oauth_show_success_message();
		} else if ( isset( $_POST['mo_oauth_server_feedback'] ) and $_POST['mo_oauth_server_feedback'] == 'true' ) {
			$user = wp_get_current_user();
			$message = 'Plugin Deactivated:';
			$deactivate_reason         = array_key_exists( 'deactivate_reason_radio', $_POST ) ? $_POST['deactivate_reason_radio'] : false;
			$deactivate_reason_message = array_key_exists( 'query_feedback', $_POST ) ? $_POST['query_feedback'] : false;
			if ( $deactivate_reason ) {
				$message .= $deactivate_reason;
				if ( isset( $deactivate_reason_message ) ) {
					$message .= ':' . $deactivate_reason_message;
				}
				$email = get_option( "mo_oauth_admin_email" );
				if ( $email == '' ) {
					$email = $user->user_email;
				}
				$phone = get_option( 'mo_oauth_server_admin_phone' );
				//only reason
				$feedback_reasons = new Mo_Auth_Server_Customer();
				$submited = json_decode( $feedback_reasons->mo_oauth_send_email_alert( $email, $phone, $message ), true );
				deactivate_plugins( __FILE__ );
				update_option( 'message', 'Thank you for the feedback.' );
				$this->mo_oauth_show_success_message();
			} else {
				update_option( 'message', 'Please Select one of the reasons ,if your reason is not mentioned please select Other Reasons' );
				$this->mo_oauth_show_error_message();
			}
		}
		// Request for demo functionality
		else if( isset( $_POST['option'] ) and $_POST['option'] == "mo_oauth_server_demo_request_form" ) {
			if( mo_oauth_server_is_curl_installed() == 0 ) {
				return $this->mo_oauth_show_curl_error();
			}
			// Demo Request
			$email = $_POST['mo_oauth_server_demo_email'];
			$demo_plan = $_POST['mo_oauth_server_demo_plan'];
			$query = $_POST['mo_oauth_server_demo_usecase'];

			$customer = new Mo_Auth_Server_Customer();
			if ( $this->mo_oauth_check_empty_or_null( $email ) || $this->mo_oauth_check_empty_or_null( $demo_plan ) || $this->mo_oauth_check_empty_or_null($query) ) {
				update_option('message', 'Please fill up Email field to submit your query.', false);
				$this->mo_oauth_show_error_message();
			} else {
				$submited = json_decode( $customer->mo_oauth_send_demo_alert( $email, $demo_plan, $query, "WP OAuth Server On Demo Request - ".$email ), true );
				update_option('message', 'Thanks for getting in touch! We shall get back to you shortly.', false);
				$this->mo_oauth_show_success_message();
			}
		}
	}

	function mo_oauth_get_current_Mo_Auth_Server_Customer(){
		$customer = new Mo_Auth_Server_Customer();
		$content = $customer->get_customer_key();
		$customerKey = json_decode( $content, true );
		if( json_last_error() == JSON_ERROR_NONE ) {
			update_option( 'mo_oauth_server_admin_customer_key', $customerKey['id'], false );
			update_option( 'mo_oauth_server_admin_api_key', $customerKey['apiKey'], false );
			update_option( 'mo_oauth_server_customer_token', $customerKey['token'], false );
			update_option( 'password', '' , false);
			update_option( 'message', 'Customer retrieved successfully', false );
			update_option('mo_oauth_server_registration_status','MO_OAUTH_CUSTOMER_RETRIEVED', false);
			delete_option('mo_oauth_server_verify_customer');
			delete_option('mo_oauth_server_new_registration');
			$this->mo_oauth_show_success_message();
			//mo_oauth_server_page();
		} else {
			update_option( 'message', 'You already have an account with miniOrange. Please enter a valid password.', false);
			update_option('mo_oauth_server_verify_customer', 'true', false);
			delete_option('mo_oauth_server_new_registration');
			//mo_oauth_server_page();
			$this->mo_oauth_show_error_message();

		}
	}

	function create_customer(){
		$customer = new Mo_Auth_Server_Customer();
		$customerKey = json_decode( $customer->create_customer(), true );
		if( strcasecmp( $customerKey['status'], 'CUSTOMER_USERNAME_ALREADY_EXISTS') == 0 ) {
			$this->mo_oauth_get_current_Mo_Auth_Server_Customer();
			delete_option('mo_oauth_server_new_customer');
		} else if( strcasecmp( $customerKey['status'], 'SUCCESS' ) == 0 ) {
			update_option( 'mo_oauth_server_admin_customer_key', $customerKey['id'], false );
			update_option( 'mo_oauth_server_admin_api_key', $customerKey['apiKey'], false );
			update_option( 'mo_oauth_server_customer_token', $customerKey['token'] , false);
			update_option( 'password', '', false);
			update_option( 'message', 'Registered successfully.', false);
			update_option('mo_oauth_server_registration_status','MO_OAUTH_REGISTRATION_COMPLETE', false);
			update_option('mo_oauth_server_new_customer',1, false);
			delete_option('mo_oauth_server_verify_customer');
			delete_option('mo_oauth_server_new_registration');
			$this->mo_oauth_show_success_message();
		}
	}

	function mo_oauth_show_curl_error() {
		if( mo_oauth_server_is_curl_installed() == 0 ) {
			update_option( 'message', '<a href="http://php.net/manual/en/curl.installation.php" target="_blank">PHP CURL extension</a> is not installed or disabled. Please enable it to continue.', false);
			$this->mo_oauth_show_error_message();
			return;
		}
	}

	function mo_oauth_shortcode_login(){
		$mowidget = new Mo_Oauth_Widget;
		$mowidget->mo_oauth_login_form();
	}

}


	function mo_oauth_server_is_customer_registered() {
		$email 			= get_option('mo_oauth_admin_email');
		$customerKey 	= get_option('mo_oauth_server_admin_customer_key');
		if( ! $email || ! $customerKey || ! is_numeric( trim( $customerKey ) ) ) {
			return 0;
		} else {
			return 1;
		}
	}

	function mo_oauth_server_is_curl_installed() {
		if  (in_array  ('curl', get_loaded_extensions())) {
			return 1;
		} else {
			return 0;
		}
	}

	function moosGenerateRandomString($length = 10) {
		$characters = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
		$charactersLength = strlen($characters);
		$randomString = '';
		for ($i = 0; $i < $length; $i++) {
			$randomString .= $characters[random_int(0, $charactersLength - 1)];
		}
		return $randomString;
	}

	/**
	 * Function to register all routes with WordPress.
	 */
	function wp_register_endpoints() {
		require( 'endpoints/registry.php' );
	    $default_routes = get_default_routes();
	    $new_routes = apply_filters( 'mo_oauth_server_define_routes', $default_routes );
	    if ( ! empty( $new_routes ) ) {
	        $default_routes = array_merge( $new_routes, $default_routes );
	    }
	    foreach( $default_routes as $route => $args ) {
	        register_rest_route( 'moserver', $route, $args );
	    }
	}

	add_action( 'rest_api_init', 'wp_register_endpoints' );

new mo_oauth_server;
